@@ -1138,6 +1138,7 @@ seahorse_gpgme_key_op_set_expires (SeahorseGpgmeSubkey *subkey,
     GDateTime *old_expires;
     ExpireParm exp_parm;
     SeahorseEditParm *parms;
+    SeahorsePgpKey *parent_key;
     gpgme_key_t key;
 
     g_return_val_if_fail (SEAHORSE_GPGME_IS_SUBKEY (subkey), GPG_E (GPG_ERR_WRONG_KEY_USAGE));
@@ -1149,7 +1150,8 @@ seahorse_gpgme_key_op_set_expires (SeahorseGpgmeSubkey *subkey,
         g_return_val_if_fail (!g_date_time_equal (old_expires, expires),
                               GPG_E (GPG_ERR_INV_VALUE));
 
-    key = seahorse_gpgme_subkey_get_pubkey (subkey);
+    parent_key = seahorse_pgp_subkey_get_parent_key (SEAHORSE_PGP_SUBKEY (subkey));
+    key = seahorse_gpgme_key_get_public (SEAHORSE_GPGME_KEY (parent_key));
     g_return_val_if_fail (key, GPG_E (GPG_ERR_INV_VALUE));
 
     exp_parm.index = seahorse_pgp_subkey_get_index (SEAHORSE_PGP_SUBKEY (subkey));
@@ -1376,6 +1378,7 @@ seahorse_gpgme_key_op_add_subkey_async (SeahorseGpgmeKey    *pkey,
     gpgme_key_t key;
     const char *algo;
     g_autofree char *algo_full = NULL;
+    int64_t expires_ts;
     g_autoptr(GSource) gsource = NULL;
     unsigned int flags = 0;
 
@@ -1403,8 +1406,12 @@ seahorse_gpgme_key_op_add_subkey_async (SeahorseGpgmeKey    *pkey,
     algo_full = g_strdup_printf ("%s%u", algo, length);
 
     /* 0 means "no expire" for us (GPGME picks a default otherwise) */
-    if (expires == 0)
+    if (expires) {
+        expires_ts = g_date_time_to_unix (expires);
+    } else {
+        expires_ts = 0;
         flags |= GPGME_CREATE_NOEXPIRE;
+    }
 
     /* Add usage flags */
     switch (type) {
@@ -1423,7 +1430,7 @@ seahorse_gpgme_key_op_add_subkey_async (SeahorseGpgmeKey    *pkey,
                                             key,
                                             algo_full,
                                             0,
-                                            g_date_time_to_unix (expires),
+                                            expires_ts,
                                             flags);
     }
 
@@ -1555,13 +1562,15 @@ del_key_transit (unsigned int   current_state,
 gpgme_error_t
 seahorse_gpgme_key_op_del_subkey (SeahorseGpgmeSubkey *subkey)
 {
-    SeahorseEditParm *parms;
+    SeahorsePgpKey *parent_key;
     gpgme_key_t key;
+    SeahorseEditParm *parms;
     int index;
 
     g_return_val_if_fail (SEAHORSE_GPGME_IS_SUBKEY (subkey), GPG_E (GPG_ERR_WRONG_KEY_USAGE));
 
-    key = seahorse_gpgme_subkey_get_pubkey (subkey);
+    parent_key = seahorse_pgp_subkey_get_parent_key (SEAHORSE_PGP_SUBKEY (subkey));
+    key = seahorse_gpgme_key_get_public (SEAHORSE_GPGME_KEY (parent_key));
     g_return_val_if_fail (key, GPG_E (GPG_ERR_INV_VALUE));
 
     index = seahorse_pgp_subkey_get_index (SEAHORSE_PGP_SUBKEY (subkey));
@@ -1738,6 +1747,7 @@ seahorse_gpgme_key_op_revoke_subkey (SeahorseGpgmeSubkey *subkey, SeahorseRevoke
     RevSubkeyParm rev_parm;
     SeahorseEditParm *parms;
     gpgme_subkey_t gsubkey;
+    SeahorsePgpKey *parent_key;
     gpgme_key_t key;
 
     g_return_val_if_fail (SEAHORSE_GPGME_IS_SUBKEY (subkey), GPG_E (GPG_ERR_WRONG_KEY_USAGE));
@@ -1745,7 +1755,8 @@ seahorse_gpgme_key_op_revoke_subkey (SeahorseGpgmeSubkey *subkey, SeahorseRevoke
     gsubkey = seahorse_gpgme_subkey_get_subkey (subkey);
     g_return_val_if_fail (!gsubkey->revoked, GPG_E (GPG_ERR_INV_VALUE));
 
-    key = seahorse_gpgme_subkey_get_pubkey (subkey);
+    parent_key = seahorse_pgp_subkey_get_parent_key (SEAHORSE_PGP_SUBKEY (subkey));
+    key = seahorse_gpgme_key_get_public (SEAHORSE_GPGME_KEY (parent_key));
     g_return_val_if_fail (key, GPG_E (GPG_ERR_INV_VALUE));
 
     rev_parm.index = seahorse_pgp_subkey_get_index (SEAHORSE_PGP_SUBKEY (subkey));
