@@ -38,13 +38,15 @@ G_BEGIN_DECLS
 #define SEAHORSE_PGP_TYPE_STR  SEAHORSE_PGP_NAME
 #define SEAHORSE_PGP           g_quark_from_string (SEAHORSE_PGP_NAME)
 
-void       seahorse_pgp_backend_initialize    (void);
+void       seahorse_pgp_backend_initialize    (const char *gpg_homedir);
 
 #define SEAHORSE_PGP_TYPE_BACKEND (seahorse_pgp_backend_get_type ())
 G_DECLARE_FINAL_TYPE (SeahorsePgpBackend, seahorse_pgp_backend, SEAHORSE_PGP, BACKEND, GObject)
 
 SeahorsePgpBackend *   seahorse_pgp_backend_get                  (void);
 
+const char *           seahorse_pgp_backend_get_gpg_homedir      (SeahorsePgpBackend *self);
+
 SeahorseGpgmeKeyring * seahorse_pgp_backend_get_default_keyring  (SeahorsePgpBackend *self);
 
 SeahorsePgpKey *       seahorse_pgp_backend_get_default_key      (SeahorsePgpBackend *self);
@@ -53,15 +55,17 @@ SeahorsePgpKey *       seahorse_pgp_backend_get_default_key      (SeahorsePgpBac
 
 SeahorseDiscovery *    seahorse_pgp_backend_get_discovery        (SeahorsePgpBackend *self);
 
+GListModel *           seahorse_pgp_backend_get_remotes          (SeahorsePgpBackend *self);
+
 SeahorseServerSource * seahorse_pgp_backend_lookup_remote        (SeahorsePgpBackend *self,
                                                                   const gchar *uri);
 
 void                   seahorse_pgp_backend_add_remote           (SeahorsePgpBackend *self,
-                                                                  const gchar *uri,
-                                                                  SeahorseServerSource *source);
+                                                                  const char         *uri,
+                                                                  gboolean            persist);
 
 void                   seahorse_pgp_backend_remove_remote        (SeahorsePgpBackend *self,
-                                                                  const gchar *uri);
+                                                                  const char         *uri);
 
 void                   seahorse_pgp_backend_search_remote_async  (SeahorsePgpBackend *self,
                                                                   const gchar *search,
