@@ -78,29 +78,30 @@ seahorse_pgp_keyid_hash (gconstpointer v)
     return g_str_hash (keyid);
 }
 
+/**
+ * seahorse_pgp_keyid_equal:
+ * @v1: A fingerprint or key id
+ * @v2: A fingerprint or key id
+ *
+ * Compares key IDs, regardless of casing.
+ */
 gboolean
-seahorse_pgp_keyid_equal (gconstpointer v1,
-                          gconstpointer v2)
+seahorse_pgp_keyid_equal (const void *v1,
+                          const void *v2)
 {
     const char *keyid_1 = v1;
     const char *keyid_2 = v2;
-    gsize len_1 = strlen (keyid_1);
-    gsize len_2 = strlen (keyid_2);
+    size_t len_1 = strlen (keyid_1);
+    size_t len_2 = strlen (keyid_2);
 
-    if (len_1 != len_2 && len_1 >= 8 && len_2 >= 8) {
-        keyid_1 += len_1 - 8;
-        keyid_2 += len_2 - 8;
+    /* In case we have a key ids of different length, go to the shortest
+     * reliable form: the long key id (16 characters) */
+    if (len_1 != len_2 && len_1 >= 16 && len_2 >= 16) {
+        keyid_1 += len_1 - 16;
+        keyid_2 += len_2 - 16;
     }
-    return g_str_equal (keyid_1, keyid_2);
-}
 
-static const char*
-calc_short_name (SeahorsePgpKey *self)
-{
-    SeahorsePgpKeyPrivate *priv = seahorse_pgp_key_get_instance_private (self);
-    g_autoptr(SeahorsePgpUid) uid = g_list_model_get_item (priv->uids, 0);
-
-    return uid ? seahorse_pgp_uid_get_name (uid) : NULL;
+    return g_ascii_strcasecmp (keyid_1, keyid_2) == 0;
 }
 
 static char*
@@ -247,7 +248,7 @@ seahorse_pgp_key_realize (SeahorsePgpKey *self)
 
     name = calc_name (self);
     markup = calc_markup (self);
-    nickname = calc_short_name (self);
+    nickname = seahorse_pgp_key_get_primary_name (self);
 
     g_object_get (self, "usage", &usage, NULL);
 
@@ -311,6 +312,18 @@ seahorse_pgp_key_get_uids (SeahorsePgpKey *self)
     return priv->uids;
 }
 
+const char *
+seahorse_pgp_key_get_primary_name (SeahorsePgpKey *self)
+{
+    SeahorsePgpKeyPrivate *priv = seahorse_pgp_key_get_instance_private (self);
+    g_autoptr(SeahorsePgpUid) uid = NULL;
+
+    g_return_val_if_fail (SEAHORSE_PGP_IS_KEY (self), NULL);
+
+    uid = g_list_model_get_item (priv->uids, 0);
+    return uid ? seahorse_pgp_uid_get_name (uid) : NULL;
+}
+
 void
 seahorse_pgp_key_add_uid (SeahorsePgpKey *self,
                           SeahorsePgpUid *uid)
@@ -380,6 +393,7 @@ seahorse_pgp_key_add_subkey (SeahorsePgpKey    *self,
             return;
     }
 
+    seahorse_pgp_subkey_set_parent_key (subkey, self);
     g_list_store_append (G_LIST_STORE (priv->subkeys), subkey);
 }
 
@@ -546,32 +560,35 @@ gboolean
 seahorse_pgp_key_has_keyid (SeahorsePgpKey *self, const char *match)
 {
     SeahorsePgpKeyPrivate *priv = seahorse_pgp_key_get_instance_private (self);
-    guint n_match;
 
     g_return_val_if_fail (SEAHORSE_PGP_IS_KEY (self), FALSE);
-    g_return_val_if_fail (match, FALSE);
-
-    n_match = strlen (match);
+    g_return_val_if_fail (match && *match, FALSE);
 
     for (guint i = 0; i < g_list_model_get_n_items (priv->subkeys); i++) {
         g_autoptr(SeahorsePgpSubkey) subkey = NULL;
         const char *keyid;
-        guint n_keyid;
 
         subkey = g_list_model_get_item (priv->subkeys, i);
         keyid = seahorse_pgp_subkey_get_keyid (subkey);
         g_return_val_if_fail (keyid, FALSE);
-        n_keyid = strlen (keyid);
-        if (n_match <= n_keyid) {
-            keyid += (n_keyid - n_match);
-            if (strncmp (keyid, match, n_match) == 0)
-                return TRUE;
-        }
+        if (seahorse_pgp_keyid_equal (keyid, match))
+            return TRUE;
     }
 
     return FALSE;
 }
 
+gboolean
+seahorse_pgp_key_is_private_key (SeahorsePgpKey *self)
+{
+    SeahorseUsage usage;
+
+    g_return_val_if_fail (SEAHORSE_PGP_IS_KEY (self), FALSE);
+
+    usage = seahorse_object_get_usage (SEAHORSE_OBJECT (self));
+    return usage == SEAHORSE_USAGE_PRIVATE_KEY;
+}
+
 static void
 seahorse_pgp_key_init (SeahorsePgpKey *self)
 {
