@@ -29,6 +29,7 @@
 
 enum {
     PROP_0,
+    PROP_PARENT_KEY,
     PROP_INDEX,
     PROP_KEYID,
     PROP_FLAGS,
@@ -43,271 +44,366 @@ enum {
 static GParamSpec *obj_props[N_PROPS] = { NULL, };
 
 typedef struct _SeahorsePgpSubkeyPrivate {
-    guint index;
-    gchar *keyid;
-    guint flags;
-    guint length;
-    gchar *algorithm;
-    gulong created;
-    gulong expires;
-    gchar *description;
-    gchar *fingerprint;
+    SeahorsePgpKey *parent_key;
+    unsigned int index;
+    char *keyid;
+    unsigned int flags;
+    unsigned int length;
+    char *algorithm;
+    GDateTime *created;
+    GDateTime *expires;
+    char *description;
+    char *fingerprint;
 } SeahorsePgpSubkeyPrivate;
 
 G_DEFINE_TYPE_WITH_PRIVATE (SeahorsePgpSubkey, seahorse_pgp_subkey, G_TYPE_OBJECT);
 
-guint
+SeahorsePgpKey *
+seahorse_pgp_subkey_get_parent_key (SeahorsePgpSubkey *self)
+{
+    SeahorsePgpSubkeyPrivate *priv = seahorse_pgp_subkey_get_instance_private (self);
+
+    g_return_val_if_fail (SEAHORSE_PGP_IS_SUBKEY (self), NULL);
+    return priv->parent_key;
+}
+
+void
+seahorse_pgp_subkey_set_parent_key (SeahorsePgpSubkey *self,
+                                    SeahorsePgpKey    *parent_key)
+{
+    SeahorsePgpSubkeyPrivate *priv = seahorse_pgp_subkey_get_instance_private (self);
+
+    g_return_if_fail (SEAHORSE_PGP_IS_SUBKEY (self));
+    g_return_if_fail (SEAHORSE_PGP_IS_KEY (parent_key));
+
+    priv->parent_key = parent_key;
+    g_object_notify_by_pspec (G_OBJECT (self), obj_props[PROP_PARENT_KEY]);
+}
+
+unsigned int
 seahorse_pgp_subkey_get_index (SeahorsePgpSubkey *self)
 {
-    SeahorsePgpSubkeyPrivate *priv;
+    SeahorsePgpSubkeyPrivate *priv = seahorse_pgp_subkey_get_instance_private (self);
 
     g_return_val_if_fail (SEAHORSE_PGP_IS_SUBKEY (self), 0);
-
-    priv = seahorse_pgp_subkey_get_instance_private (self);
     return priv->index;
 }
 
 void
-seahorse_pgp_subkey_set_index (SeahorsePgpSubkey *self, guint index)
+seahorse_pgp_subkey_set_index (SeahorsePgpSubkey *self, unsigned int index)
 {
-    SeahorsePgpSubkeyPrivate *priv;
+    SeahorsePgpSubkeyPrivate *priv = seahorse_pgp_subkey_get_instance_private (self);
 
     g_return_if_fail (SEAHORSE_PGP_IS_SUBKEY (self));
 
-    priv = seahorse_pgp_subkey_get_instance_private (self);
     priv->index = index;
     g_object_notify_by_pspec (G_OBJECT (self), obj_props[PROP_INDEX]);
 }
 
-const gchar*
+const char *
 seahorse_pgp_subkey_get_keyid (SeahorsePgpSubkey *self)
 {
-    SeahorsePgpSubkeyPrivate *priv;
+    SeahorsePgpSubkeyPrivate *priv = seahorse_pgp_subkey_get_instance_private (self);
 
     g_return_val_if_fail (SEAHORSE_PGP_IS_SUBKEY (self), NULL);
-
-    priv = seahorse_pgp_subkey_get_instance_private (self);
     return priv->keyid;
 }
 
 void
-seahorse_pgp_subkey_set_keyid (SeahorsePgpSubkey *self, const gchar *keyid)
+seahorse_pgp_subkey_set_keyid (SeahorsePgpSubkey *self, const char *keyid)
 {
-    SeahorsePgpSubkeyPrivate *priv;
+    SeahorsePgpSubkeyPrivate *priv = seahorse_pgp_subkey_get_instance_private (self);
 
     g_return_if_fail (SEAHORSE_PGP_IS_SUBKEY (self));
+    g_return_if_fail (keyid);
 
-    priv = seahorse_pgp_subkey_get_instance_private (self);
     g_free (priv->keyid);
     priv->keyid = g_strdup (keyid);
     g_object_notify_by_pspec (G_OBJECT (self), obj_props[PROP_KEYID]);
 }
 
-guint
+unsigned int
 seahorse_pgp_subkey_get_flags (SeahorsePgpSubkey *self)
 {
-    SeahorsePgpSubkeyPrivate *priv;
+    SeahorsePgpSubkeyPrivate *priv = seahorse_pgp_subkey_get_instance_private (self);
 
     g_return_val_if_fail (SEAHORSE_PGP_IS_SUBKEY (self), 0);
-
-    priv = seahorse_pgp_subkey_get_instance_private (self);
     return priv->flags;
 }
 
 void
-seahorse_pgp_subkey_set_flags (SeahorsePgpSubkey *self, guint flags)
+seahorse_pgp_subkey_set_flags (SeahorsePgpSubkey *self, unsigned int flags)
 {
-    SeahorsePgpSubkeyPrivate *priv;
+    SeahorsePgpSubkeyPrivate *priv = seahorse_pgp_subkey_get_instance_private (self);
 
     g_return_if_fail (SEAHORSE_PGP_IS_SUBKEY (self));
 
-    priv = seahorse_pgp_subkey_get_instance_private (self);
     priv->flags = flags;
     g_object_notify_by_pspec (G_OBJECT (self), obj_props[PROP_FLAGS]);
 }
 
-guint
+/**
+ * seahorse_pgp_subkey_get_length:
+ * @self: A #SeahorsePgpSubkey
+ *
+ * Returns: the subkey's length
+ */
+unsigned int
 seahorse_pgp_subkey_get_length (SeahorsePgpSubkey *self)
 {
-    SeahorsePgpSubkeyPrivate *priv;
+    SeahorsePgpSubkeyPrivate *priv = seahorse_pgp_subkey_get_instance_private (self);
 
     g_return_val_if_fail (SEAHORSE_PGP_IS_SUBKEY (self), 0);
-
-    priv = seahorse_pgp_subkey_get_instance_private (self);
     return priv->length;
 }
 
+/**
+ * seahorse_pgp_subkey_set_length:
+ * @self: A #SeahorsePgpSubkey
+ * @length: The new key length
+ */
 void
-seahorse_pgp_subkey_set_length (SeahorsePgpSubkey *self, guint length)
+seahorse_pgp_subkey_set_length (SeahorsePgpSubkey *self, unsigned int length)
 {
-    SeahorsePgpSubkeyPrivate *priv;
+    SeahorsePgpSubkeyPrivate *priv = seahorse_pgp_subkey_get_instance_private (self);
 
     g_return_if_fail (SEAHORSE_PGP_IS_SUBKEY (self));
 
-    priv = seahorse_pgp_subkey_get_instance_private (self);
     priv->length = length;
     g_object_notify_by_pspec (G_OBJECT (self), obj_props[PROP_LENGTH]);
 }
 
-gchar *
+/**
+ * seahorse_pgp_subkey_get_usage:
+ * @self: A #SeahorsePgpSubkey
+ *
+ * Returns: (transfer full): the subkey's usage, e.g. "Encrypt, Sign, Certify"
+ */
+char *
 seahorse_pgp_subkey_get_usage (SeahorsePgpSubkey *self)
+{
+    g_auto(GStrv) usages = NULL;
+
+    g_return_val_if_fail (SEAHORSE_PGP_IS_SUBKEY (self), NULL);
+
+    usages = seahorse_pgp_subkey_get_usages (self, NULL);
+    return g_strjoinv (", ", usages);
+}
+
+/**
+ * seahorse_pgp_subkey_get_usages:
+ * @self: A #SeahorsePgpSubkey
+ * @decriptions: (transfer full) (array zero-terminated=1) (optional):
+ *               The descriptions of the subkey's usages
+ *
+ * Returns: (transfer full) (array zero-terminated=1): the subkey's usages,
+ * for example: { "Encrypt" , "Sign" , "Certify", %NULL }
+ */
+char **
+seahorse_pgp_subkey_get_usages (SeahorsePgpSubkey *self, char ***descriptions)
 {
     typedef struct {
         unsigned int flag;
         const char *name;
+        const char *description;
     } FlagNames;
 
     const FlagNames flag_names[] = {
-        { SEAHORSE_FLAG_CAN_ENCRYPT,      N_("Encrypt") },
-        { SEAHORSE_FLAG_CAN_SIGN,         N_("Sign") },
-        { SEAHORSE_FLAG_CAN_CERTIFY,      N_("Certify") },
-        { SEAHORSE_FLAG_CAN_AUTHENTICATE, N_("Authenticate") }
+        { SEAHORSE_FLAG_CAN_ENCRYPT, N_("Encrypt"), N_("This subkey can be used for encryption") },
+        { SEAHORSE_FLAG_CAN_SIGN, N_("Sign"), N_("This subkey can be used to create data signatures") },
+        { SEAHORSE_FLAG_CAN_CERTIFY, N_("Certify"), N_("This subkey can be used to create certificates") },
+        { SEAHORSE_FLAG_CAN_AUTHENTICATE, N_("Authenticate"), N_("This subkey can be used for authentication") }
     };
 
-    SeahorsePgpSubkeyPrivate *priv;
-    GString *str;
-    gboolean previous;
-    uint i;
+    SeahorsePgpSubkeyPrivate *priv = seahorse_pgp_subkey_get_instance_private (self);
+    g_autoptr(GPtrArray) names = NULL, descs = NULL;
 
     g_return_val_if_fail (SEAHORSE_PGP_IS_SUBKEY (self), NULL);
 
-    priv = seahorse_pgp_subkey_get_instance_private (self);
-    str = g_string_new (NULL);
-    previous = FALSE;
+    names = g_ptr_array_new_with_free_func (g_free);
+    descs = g_ptr_array_new_with_free_func (g_free);
 
-    for (i = 0; i < G_N_ELEMENTS (flag_names); i++) {
+    for (unsigned i = 0; i < G_N_ELEMENTS (flag_names); i++) {
         if (priv->flags & flag_names[i].flag) {
-            if (previous)
-                g_string_append (str, ", ");
-
-            previous = TRUE;
-            g_string_append (str, _(flag_names[i].name));
+            g_ptr_array_add (names, g_strdup (_(flag_names[i].name)));
+            g_ptr_array_add (descs, g_strdup (_(flag_names[i].description)));
         }
     }
 
-    return g_string_free (str, FALSE);
+    g_ptr_array_add (names, NULL);
+    g_ptr_array_add (descs, NULL);
+
+    if (descriptions)
+        *descriptions = (char**) g_ptr_array_free (g_steal_pointer (&descs), FALSE);
+
+    return (char **) g_ptr_array_free (g_steal_pointer (&names), FALSE);
 }
 
-const gchar*
+/**
+ * seahorse_pgp_subkey_get_algorithm:
+ * @self: A #SeahorsePgpSubkey
+ *
+ * Returns: (transfer none): the subkey's algorithm
+ */
+const char *
 seahorse_pgp_subkey_get_algorithm (SeahorsePgpSubkey *self)
 {
-    SeahorsePgpSubkeyPrivate *priv;
+    SeahorsePgpSubkeyPrivate *priv = seahorse_pgp_subkey_get_instance_private (self);
 
     g_return_val_if_fail (SEAHORSE_PGP_IS_SUBKEY (self), NULL);
-
-    priv = seahorse_pgp_subkey_get_instance_private (self);
     return priv->algorithm;
 }
 
+/**
+ * seahorse_pgp_subkey_set_algorithm:
+ * @self: A #SeahorsePgpSubkey
+ * @algorithm: An algorithm, for example "RSA"
+ *
+ * Sets the algorithm of @self.
+ */
 void
-seahorse_pgp_subkey_set_algorithm (SeahorsePgpSubkey *self, const gchar *algorithm)
+seahorse_pgp_subkey_set_algorithm (SeahorsePgpSubkey *self,
+                                   const char        *algorithm)
 {
-    SeahorsePgpSubkeyPrivate *priv;
+    SeahorsePgpSubkeyPrivate *priv = seahorse_pgp_subkey_get_instance_private (self);
 
     g_return_if_fail (SEAHORSE_PGP_IS_SUBKEY (self));
 
-    priv = seahorse_pgp_subkey_get_instance_private (self);
     g_free (priv->algorithm);
     priv->algorithm = g_strdup (algorithm);
     g_object_notify_by_pspec (G_OBJECT (self), obj_props[PROP_ALGORITHM]);
 }
 
-gulong
+/**
+ * seahorse_pgp_subkey_get_created:
+ * @self: A #SeahorsePgpSubkey
+ *
+ * Returns the #GDateTime at which this key was created, or %NULL if unknown.
+ *
+ * Returns: (transfer none) (nullable): the creation datetime, or %NULL
+ */
+GDateTime *
 seahorse_pgp_subkey_get_created (SeahorsePgpSubkey *self)
 {
-    SeahorsePgpSubkeyPrivate *priv;
+    SeahorsePgpSubkeyPrivate *priv = seahorse_pgp_subkey_get_instance_private (self);
 
     g_return_val_if_fail (SEAHORSE_PGP_IS_SUBKEY (self), 0);
-
-    priv = seahorse_pgp_subkey_get_instance_private (self);
     return priv->created;
 }
 
+/**
+ * seahorse_pgp_subkey_set_created:
+ * @self: A #SeahorsePgpSubkey
+ * @expires: (nullable): The datetime @self was create or %NULL if unknown
+ *
+ * Sets the creation datetime of @self (can be %NULL, if unknown).
+ */
 void
-seahorse_pgp_subkey_set_created (SeahorsePgpSubkey *self, gulong created)
+seahorse_pgp_subkey_set_created (SeahorsePgpSubkey *self,
+                                 GDateTime         *created)
 {
-    SeahorsePgpSubkeyPrivate *priv;
+    SeahorsePgpSubkeyPrivate *priv = seahorse_pgp_subkey_get_instance_private (self);
 
     g_return_if_fail (SEAHORSE_PGP_IS_SUBKEY (self));
 
-    priv = seahorse_pgp_subkey_get_instance_private (self);
-    priv->created = created;
+    g_clear_pointer (&priv->created, g_date_time_unref);
+    priv->created = created? g_date_time_ref (created) : NULL;
     g_object_notify_by_pspec (G_OBJECT (self), obj_props[PROP_CREATED]);
 }
 
-gulong
+/**
+ * seahorse_pgp_subkey_get_expires:
+ * @self: A #SeahorsePgpSubkey
+ *
+ * Returns the #GDateTime at which this key will expire, or %NULL if not set.
+ *
+ * Returns: (transfer none) (nullable): the expiration datetime, or %NULL
+ */
+GDateTime *
 seahorse_pgp_subkey_get_expires (SeahorsePgpSubkey *self)
 {
-    SeahorsePgpSubkeyPrivate *priv;
+    SeahorsePgpSubkeyPrivate *priv = seahorse_pgp_subkey_get_instance_private (self);
 
     g_return_val_if_fail (SEAHORSE_PGP_IS_SUBKEY (self), 0);
 
-    priv = seahorse_pgp_subkey_get_instance_private (self);
     return priv->expires;
 }
 
+/**
+ * seahorse_pgp_subkey_set_expires:
+ * @self: A #SeahorsePgpSubkey
+ * @expires: (nullable): The expiration datetime or %NULL if none.
+ *
+ * Sets the expiration datetime of @self (or if %NULL, subkey doesn't expire).
+ */
 void
-seahorse_pgp_subkey_set_expires (SeahorsePgpSubkey *self, gulong expires)
+seahorse_pgp_subkey_set_expires (SeahorsePgpSubkey *self,
+                                 GDateTime         *expires)
 {
-    SeahorsePgpSubkeyPrivate *priv;
+    SeahorsePgpSubkeyPrivate *priv = seahorse_pgp_subkey_get_instance_private (self);
 
     g_return_if_fail (SEAHORSE_PGP_IS_SUBKEY (self));
 
-    priv = seahorse_pgp_subkey_get_instance_private (self);
-    priv->expires = expires;
+    g_clear_pointer (&priv->expires, g_date_time_unref);
+    priv->expires = expires? g_date_time_ref (expires) : NULL;
     g_object_notify_by_pspec (G_OBJECT (self), obj_props[PROP_EXPIRES]);
 }
 
-const gchar*
+const char *
 seahorse_pgp_subkey_get_fingerprint (SeahorsePgpSubkey *self)
 {
-    SeahorsePgpSubkeyPrivate *priv;
+    SeahorsePgpSubkeyPrivate *priv = seahorse_pgp_subkey_get_instance_private (self);
 
     g_return_val_if_fail (SEAHORSE_PGP_IS_SUBKEY (self), NULL);
-
-    priv = seahorse_pgp_subkey_get_instance_private (self);
     return priv->fingerprint;
 }
 
+/**
+ * seahorse_pgp_subkey_set_expires:
+ * @self: A #SeahorsePgpSubkey
+ * @fingerprint: The fingerprint of @key
+ *
+ * Sets the fingerprint of @self to @fingerprint
+ */
 void
-seahorse_pgp_subkey_set_fingerprint (SeahorsePgpSubkey *self, const gchar *fingerprint)
+seahorse_pgp_subkey_set_fingerprint (SeahorsePgpSubkey *self,
+                                     const char        *fingerprint)
 {
-    SeahorsePgpSubkeyPrivate *priv;
+    SeahorsePgpSubkeyPrivate *priv = seahorse_pgp_subkey_get_instance_private (self);
 
     g_return_if_fail (SEAHORSE_PGP_IS_SUBKEY (self));
+    g_return_if_fail (fingerprint);
 
-    priv = seahorse_pgp_subkey_get_instance_private (self);
     g_free (priv->fingerprint);
     priv->fingerprint = g_strdup (fingerprint);
     g_object_notify_by_pspec (G_OBJECT (self), obj_props[PROP_FINGERPRINT]);
 }
 
-const gchar*
+const char *
 seahorse_pgp_subkey_get_description (SeahorsePgpSubkey *self)
 {
-    SeahorsePgpSubkeyPrivate *priv;
+    SeahorsePgpSubkeyPrivate *priv = seahorse_pgp_subkey_get_instance_private (self);
 
     g_return_val_if_fail (SEAHORSE_PGP_IS_SUBKEY (self), NULL);
-
-    priv = seahorse_pgp_subkey_get_instance_private (self);
     return priv->description;
 }
 
 void
-seahorse_pgp_subkey_set_description (SeahorsePgpSubkey *self, const gchar *description)
+seahorse_pgp_subkey_set_description (SeahorsePgpSubkey *self,
+                                     const char        *description)
 {
-    SeahorsePgpSubkeyPrivate *priv;
+    SeahorsePgpSubkeyPrivate *priv = seahorse_pgp_subkey_get_instance_private (self);
 
     g_return_if_fail (SEAHORSE_PGP_IS_SUBKEY (self));
+    g_return_if_fail (description);
 
-    priv = seahorse_pgp_subkey_get_instance_private (self);
     g_free (priv->description);
     priv->description = g_strdup (description);
     g_object_notify_by_pspec (G_OBJECT (self), obj_props[PROP_DESCRIPTION]);
 }
 
-gchar*
-seahorse_pgp_subkey_calc_description (const gchar *name, guint index)
+char *
+seahorse_pgp_subkey_calc_description (const char *name, unsigned int index)
 {
     if (name == NULL)
         name = _("Key");
@@ -321,14 +417,13 @@ seahorse_pgp_subkey_calc_description (const gchar *name, guint index)
 /* Takes runs of hexadecimal digits, possibly with whitespace among them, and
  * formats them nicely in groups of four digits.
  */
-gchar*
-seahorse_pgp_subkey_calc_fingerprint (const gchar *raw_fingerprint)
+char *
+seahorse_pgp_subkey_calc_fingerprint (const char *raw_fingerprint)
 {
-    const gchar *raw;
+    const char *raw;
     GString *string;
-    guint i, len;
-    guint num_digits;
-    gchar *fpr;
+    unsigned int len, num_digits;
+    char *fpr;
 
     raw = raw_fingerprint;
     g_return_val_if_fail (raw != NULL, NULL);
@@ -337,7 +432,7 @@ seahorse_pgp_subkey_calc_fingerprint (const gchar *raw_fingerprint)
     len = strlen (raw);
 
     num_digits = 0;
-    for (i = 0; i < len; i++) {
+    for (unsigned int i = 0; i < len; i++) {
         if (g_ascii_isxdigit (raw[i])) {
             g_string_append_c (string, g_ascii_toupper (raw[i]));
             num_digits++;
@@ -356,12 +451,17 @@ seahorse_pgp_subkey_calc_fingerprint (const gchar *raw_fingerprint)
 }
 
 static void
-seahorse_pgp_subkey_get_property (GObject *object, guint prop_id,
-                                  GValue *value, GParamSpec *pspec)
+seahorse_pgp_subkey_get_property (GObject      *object,
+                                  unsigned int  prop_id,
+                                  GValue       *value,
+                                  GParamSpec   *pspec)
 {
     SeahorsePgpSubkey *self = SEAHORSE_PGP_SUBKEY (object);
 
     switch (prop_id) {
+    case PROP_PARENT_KEY:
+        g_value_set_object (value, seahorse_pgp_subkey_get_parent_key (self));
+        break;
     case PROP_INDEX:
         g_value_set_uint (value, seahorse_pgp_subkey_get_index (self));
         break;
@@ -378,10 +478,10 @@ seahorse_pgp_subkey_get_property (GObject *object, guint prop_id,
         g_value_set_string (value, seahorse_pgp_subkey_get_algorithm (self));
         break;
     case PROP_CREATED:
-        g_value_set_ulong (value, seahorse_pgp_subkey_get_created (self));
+        g_value_set_boxed (value, seahorse_pgp_subkey_get_created (self));
         break;
     case PROP_EXPIRES:
-        g_value_set_ulong (value, seahorse_pgp_subkey_get_expires (self));
+        g_value_set_boxed (value, seahorse_pgp_subkey_get_expires (self));
         break;
     case PROP_DESCRIPTION:
         g_value_set_string (value, seahorse_pgp_subkey_get_description (self));
@@ -393,12 +493,17 @@ seahorse_pgp_subkey_get_property (GObject *object, guint prop_id,
 }
 
 static void
-seahorse_pgp_subkey_set_property (GObject *object, guint prop_id, const GValue *value,
-                                  GParamSpec *pspec)
+seahorse_pgp_subkey_set_property (GObject      *object,
+                                  unsigned int  prop_id,
+                                  const GValue *value,
+                                  GParamSpec   *pspec)
 {
     SeahorsePgpSubkey *self = SEAHORSE_PGP_SUBKEY (object);
 
     switch (prop_id) {
+    case PROP_PARENT_KEY:
+        seahorse_pgp_subkey_set_parent_key (self, g_value_get_object (value));
+        break;
     case PROP_INDEX:
         seahorse_pgp_subkey_set_index (self, g_value_get_uint (value));
         break;
@@ -415,10 +520,10 @@ seahorse_pgp_subkey_set_property (GObject *object, guint prop_id, const GValue *
         seahorse_pgp_subkey_set_algorithm (self, g_value_get_string (value));
         break;
     case PROP_CREATED:
-        seahorse_pgp_subkey_set_created (self, g_value_get_ulong (value));
+        seahorse_pgp_subkey_set_created (self, g_value_get_boxed (value));
         break;
     case PROP_EXPIRES:
-        seahorse_pgp_subkey_set_expires (self, g_value_get_ulong (value));
+        seahorse_pgp_subkey_set_expires (self, g_value_get_boxed (value));
         break;
     case PROP_FINGERPRINT:
         seahorse_pgp_subkey_set_fingerprint (self, g_value_get_string (value));
@@ -436,6 +541,8 @@ seahorse_pgp_subkey_finalize (GObject *gobject)
     SeahorsePgpSubkeyPrivate *priv
         = seahorse_pgp_subkey_get_instance_private (self);
 
+    g_clear_pointer (&priv->created, g_date_time_unref);
+    g_clear_pointer (&priv->expires, g_date_time_unref);
     g_clear_pointer (&priv->algorithm, g_free);
     g_clear_pointer (&priv->fingerprint, g_free);
     g_clear_pointer (&priv->description, g_free);
@@ -458,6 +565,11 @@ seahorse_pgp_subkey_class_init (SeahorsePgpSubkeyClass *klass)
     gobject_class->set_property = seahorse_pgp_subkey_set_property;
     gobject_class->get_property = seahorse_pgp_subkey_get_property;
 
+    obj_props[PROP_PARENT_KEY] =
+        g_param_spec_object ("parent-key", "Parent Key", "Key this subkey belongs to",
+                             SEAHORSE_PGP_TYPE_KEY,
+                             G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+
     obj_props[PROP_INDEX] =
         g_param_spec_uint ("index", "Index", "PGP subkey index",
                            0, G_MAXUINT, 0,
@@ -484,13 +596,13 @@ seahorse_pgp_subkey_class_init (SeahorsePgpSubkeyClass *klass)
                              G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 
     obj_props[PROP_CREATED] =
-        g_param_spec_ulong ("created", "Created On", "Date this key was created on",
-                            0, G_MAXULONG, 0,
+        g_param_spec_boxed ("created", "Created On", "Date this key was created on",
+                            G_TYPE_DATE_TIME,
                             G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 
     obj_props[PROP_EXPIRES] =
-        g_param_spec_ulong ("expires", "Expires On", "Date this key expires on",
-                            0, G_MAXULONG, 0,
+        g_param_spec_boxed ("expires", "Expires On", "Date this key expires on",
+                            G_TYPE_DATE_TIME,
                             G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 
     obj_props[PROP_DESCRIPTION] =
