@@ -38,8 +38,6 @@
 
 #pragma once
 
-#include "seahorse-pgp-key.h"
-
 #include <gcr/gcr.h>
 
 #define SEAHORSE_TYPE_SERVER_SOURCE (seahorse_server_source_get_type ())
@@ -83,7 +81,7 @@ struct _SeahorseServerSourceClass {
 	                                          GError **error);
 };
 
-SeahorseServerSource*  seahorse_server_source_new              (const gchar *uri);
+SeahorseServerSource*  seahorse_server_source_new              (const char *uri);
 
 void                   seahorse_server_source_search_async     (SeahorseServerSource *self,
                                                                 const gchar *match,
